package org.act.od.impl.view {

import flash.events.MouseEvent;

import mx.binding.utils.BindingUtils;
import mx.containers.Box;
import mx.containers.HBox;
import mx.containers.HDividedBox;
import mx.containers.TabNavigator;
import mx.containers.VDividedBox;
import mx.controls.Button;
import mx.controls.Spacer;
import mx.core.Application;
import mx.core.ScrollPolicy;
import mx.effects.WipeRight;
import mx.events.CollectionEvent;
import mx.events.FlexEvent;
import mx.events.PropertyChangeEvent;
import mx.events.PropertyChangeEventKind;
import mx.managers.ToolTipManager;

import org.act.od.impl.view.editor.Microimage;
import org.act.od.impl.view.fileNavigator.FileNavigatorTab;
import org.act.od.impl.view.toolMenuBar.DesignerToolBar;
import org.ow2.orchestra.designer.business.PingService;
import org.ow2.orchestra.designer.events.DesignerToolBarAppEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.skins.NotificationButtonSkin;
import org.ow2.orchestra.designer.view.NotificationPanel;
import org.ow2.orchestra.designer.view.ProcessTabNavigator;
import org.ow2.orchestra.designer.view.dataMappingEditor.DataMapperView;
import org.ow2.orchestra.designer.view.dataTypeEditor.DataTypeEditorView;
import org.ow2.orchestra.designer.view.menuBar.DesignerMenuBar;
import org.ow2.orchestra.designer.view.palette.bpmn.BpmnPalette;
import org.ow2.orchestra.designer.view.properties.PropertyView;
import org.ow2.orchestra.designer.view.wsdlNavigator.WsdlNavigatorView;

/**
 * Initialize all views and Controllers
 */
public class OrchestraDesigner extends Box {

  /** Menu Bar */
  private var designerMenuBar:DesignerMenuBar;

  /** Tool Bar*/
  private var designerToolBar:DesignerToolBar;

  /** Attribute Navigator*/
  private var _propertyView:PropertyView;

  /** Data Type Editor Tab */
  private var _dataTypeView:DataTypeEditorView;

  /** Data Mapping Editor tab*/
  private var _dataMappingView:DataMapperView;

  /** Process tab navigator */
  private var processTabNavigator:ProcessTabNavigator;

  /** bpmn palette */
  private var bpmnPalette:BpmnPalette;

  /** Tab Navigators */
  private var leftTabNavigator:TabNavigator;
  private var bottomTabNavigator:TabNavigator;


  /**  Microimage */
  private var microimage:Microimage;

  /** Divided boxes */
  private var mainHDividedBox:HDividedBox = new HDividedBox();
  private var leftVDividedBox:VDividedBox = new VDividedBox();
  private var middleVDividedBox:VDividedBox = new VDividedBox();

  private var toolAndNotificationsBox:HBox = new HBox();
  private var notificationPanel:NotificationPanel = new NotificationPanel();
  private var notificationsButton:Button = new Button();

  /**
   * Initialize OrchestraDesigner.
   */
  public function OrchestraDesigner() {
    super();
    this.percentWidth = 100;
    this.percentHeight = 100;
    this.init();

  }

  /**
   * Initialize all views.
   */
  private function init():void {
    //initilize main menu
    designerMenuBar = new DesignerMenuBar();
    this.addChild(designerMenuBar);


    toolAndNotificationsBox.percentHeight = 2;
    toolAndNotificationsBox.percentWidth = 100;
    toolAndNotificationsBox.setStyle("horizontalAlign", "center");
    toolAndNotificationsBox.setStyle("verticalAlign", "middle");

    designerToolBar = new DesignerToolBar();
    toolAndNotificationsBox.addChild(designerToolBar);

    notificationsButton.setStyle("color", "0xFFFFFF");
    notificationsButton.setStyle("textAlign", "center");
    notificationsButton.setStyle("fontWeight", "bold");
    notificationsButton.setStyle("fontSize", "13");
    notificationsButton.setStyle("skin", NotificationButtonSkin);
    notificationsButton.automationName = "notificationsButton";

    BindingUtils.bindSetter(updateNotificationButton,
        DesignerModelLocator.getInstance().unreadNotifications, ["length"]);

    toolAndNotificationsBox.addChild(notificationsButton);
    var spacer:Spacer = new Spacer();
    spacer.width = 2;
    toolAndNotificationsBox.addChild(spacer);
    this.addChild(toolAndNotificationsBox);

    leftTabNavigator = new TabNavigator();
    leftTabNavigator.automationName = "leftTabNavigator";
    leftTabNavigator.autoLayout = true;
    leftTabNavigator.percentHeight = 100;
    leftTabNavigator.percentWidth = 100;

    var fileNavigatorTab:FileNavigatorTab = new FileNavigatorTab();
    leftTabNavigator.addChild(fileNavigatorTab);

    microimage = Microimage.getInstance();
    microimage.name = "microimage";


    // create process tab navigator
    processTabNavigator = new ProcessTabNavigator();
    processTabNavigator.automationName = "processTabNavigator";
    processTabNavigator.percentHeight = 75;
    processTabNavigator.percentWidth = 100;

    // create BPMN palette
    bpmnPalette = new BpmnPalette();
    bpmnPalette.automationName = "palette";
    bpmnPalette.percentHeight = 100;
    bpmnPalette.percentWidth = 100;
    leftTabNavigator.addChild(bpmnPalette);

    // create Wsdl service navigator
    var wsdlNavigator:WsdlNavigatorView = new WsdlNavigatorView();
    wsdlNavigator.automationName = "wsdlNavigatorView";
    wsdlNavigator.percentHeight = 100;
    wsdlNavigator.percentWidth = 100;
    leftTabNavigator.addChild(wsdlNavigator);

    leftTabNavigator.addEventListener(FlexEvent.CREATION_COMPLETE, leftTabComplete);

    //layout arrangement
    middleVDividedBox.name = "midVDBox";
    _propertyView = new PropertyView();
    _propertyView.automationName = "propertyView";
    _propertyView.percentHeight = 100;
    _propertyView.percentWidth = 100;

    _dataMappingView = new DataMapperView();
    _dataMappingView.automationName = "dataMappingView";
    _dataMappingView.percentHeight = 100;
    _dataMappingView.percentWidth = 100;

    _dataTypeView = new DataTypeEditorView();
    _dataTypeView.automationName = "dataTypeView";
    _dataTypeView.percentHeight = 100;
    _dataTypeView.percentWidth = 100;

    bottomTabNavigator = new TabNavigator();
    bottomTabNavigator.name = "bottomTabNavigator";
    bottomTabNavigator.automationName = "bottomTabNavigator";
    bottomTabNavigator.autoLayout = true;
    bottomTabNavigator.percentWidth = 100;
    bottomTabNavigator.percentHeight = 25;

    middleVDividedBox.addChild(processTabNavigator);
    if (DesignerModelLocator.getInstance().isPropertiesViewVisible) {
      middleVDividedBox.addChild(bottomTabNavigator);
      bottomTabNavigator.addChild(_propertyView);
      bottomTabNavigator.addChild(_dataMappingView);
      bottomTabNavigator.addChild(_dataTypeView);
    }
    middleVDividedBox.percentHeight = 100;
    middleVDividedBox.percentWidth = 75;

    mainHDividedBox.name = "bottomHDBox";
    leftVDividedBox.name = "leftVDBox";
    leftVDividedBox.percentHeight = 100;
    leftVDividedBox.addChild(leftTabNavigator);

    microimage.percentHeight = 33;
    if (DesignerModelLocator.getInstance().isMicroImageViewVisible) {
      leftVDividedBox.addChild(microimage);
    }

    mainHDividedBox.addChild(leftVDividedBox);
    mainHDividedBox.addChild(middleVDividedBox);
    mainHDividedBox.percentHeight = 95;
    mainHDividedBox.percentWidth = 100;

    this.addChild(mainHDividedBox);
    this.setStyle("headerHeight", "20");
    this.setStyle("borderThicknessTop", "2");
    this.setStyle("borderThicknessLeft", "2");
    this.setStyle("borderThicknessRight", "2");
    this.setStyle("backgroundAlpha", "0.7");

    // change DividedBox style (thiner)
    mainHDividedBox.setStyle("horizontalGap", 3);
    leftVDividedBox.setStyle("verticalGap", 3);
    middleVDividedBox.setStyle("verticalGap", 3);

    mainHDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
    mainHDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;
    leftVDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
    leftVDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;
    middleVDividedBox.horizontalScrollPolicy = ScrollPolicy.OFF;
    middleVDividedBox.verticalScrollPolicy = ScrollPolicy.OFF;

    this.initToolTip();

    // Active the Manager of session
    this.addEventListener(FlexEvent.CREATION_COMPLETE, creationCompletedHandler);
    DesignerModelLocator.getInstance().addEventListener(PropertyChangeEvent.PROPERTY_CHANGE,
        this.onModelPropertyChanged);
    // Active the tab matching to the selected process
    BindingUtils.bindSetter(setSelectedProcessEditor, DesignerModelLocator.getInstance(), "selectedProcessEditor");
    // Active the tab matching to the opened process
    DesignerModelLocator.getInstance().openedProcesses.addEventListener(CollectionEvent.COLLECTION_CHANGE,
        setOpenedProcesses);
  }

  private function updateNotificationButton(value:int):void {
    notificationsButton.label = new String(value);
    notificationsButton.toolTip = value + " new notifications";
    if (value > 0) {
      notificationsButton.setStyle("notificationStyle", "on");
    } else {
      notificationsButton.setStyle("notificationStyle", "off");
    }
    notificationsButton.invalidateDisplayList();
  }

  private function initToolTip():void {
    var wiperight:WipeRight = new WipeRight();
    wiperight.duration = 600;
    wiperight.repeatCount = 1;
    ToolTipManager.showEffect = wiperight;
  }

  private function leftTabComplete(event:FlexEvent):void {
    var tab:Button;
    var idx:uint;
    var len:uint = leftTabNavigator.numChildren;
    for (idx = 0; idx < len; idx++) {
      tab = leftTabNavigator.getTabAt(idx) as Button;
      tab.automationName = "tab" + idx;
    }
    new DesignerToolBarAppEvent(DesignerToolBarAppEvent.ADD_USERNAME).dispatch();
  }

  private function onModelPropertyChanged(event:PropertyChangeEvent):void {
    if (event.kind == PropertyChangeEventKind.UPDATE) {
      if (event.property == "isPropertiesViewVisible") {

        if (DesignerModelLocator.getInstance().isPropertiesViewVisible) {
          bottomTabNavigator.addChild(_propertyView);
          bottomTabNavigator.addChild(_dataMappingView);
          bottomTabNavigator.addChild(_dataTypeView);
          if (bottomTabNavigator.parent == null) {
            middleVDividedBox.addChild(bottomTabNavigator);
          }
        } else {
          bottomTabNavigator.removeChild(_propertyView);
          bottomTabNavigator.removeChild(_dataMappingView);
          bottomTabNavigator.removeChild(_dataTypeView);
          if (bottomTabNavigator.numChildren == 0) {
            middleVDividedBox.removeChild(bottomTabNavigator);
          }
        }
      } else if (event.property == "isMicroImageViewVisible") {

        if (DesignerModelLocator.getInstance().isMicroImageViewVisible) {
          leftVDividedBox.addChild(microimage);
        } else {
          leftVDividedBox.removeChild(microimage);
        }
      }
    }
  }

  /**
   * deactivate the button "save"
   * @param selectedProcessEditor
   */
  private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
    if (DesignerModelLocator.getInstance().selectedProcessEditor != null) {
      designerToolBar.enabledSave(DesignerModelLocator.getInstance().selectedProcessEditor.readWrite);
    } else {
      designerToolBar.enabledSave(false);
    }
  }

  /**
   *  deactivate the button "saveAll"
   * @param event
   */
  private function setOpenedProcesses(event:CollectionEvent):void {
    var value:Boolean = false;
    if (DesignerModelLocator.getInstance().openedProcesses.length > 0) {
      for each (var processEditorModel:Object in DesignerModelLocator.getInstance().openedProcesses) {
        if ((processEditorModel as ProcessEditorModel).readWrite) {
          value = true;
          break;
        }
      }
    }
    designerToolBar.enabledSaveAll(value);
  }

  /**
   * Active the manager of session
   * @param event
   */
  private function creationCompletedHandler(event:FlexEvent):void {
    PingService.getInstance().start();

    Application.application.addChild(notificationPanel);
    notificationPanel.visible = false;
    this.notificationsButton.addEventListener(MouseEvent.CLICK, displayNotificationPanel);
    this.addEventListener(MouseEvent.CLICK, removeNotificationPanel);
  }

  private function displayNotificationPanel(event:MouseEvent):void {
    event.preventDefault();
    event.stopImmediatePropagation();
    if (!notificationPanel.visible) {
      notificationPanel.x = Application.application.width - Application.application.width / 3;
      notificationPanel.y = designerMenuBar.height + toolAndNotificationsBox.height + 2;
    }
    notificationPanel.visible = ! notificationPanel.visible;
  }

  private function removeNotificationPanel(event:MouseEvent):void {
    notificationPanel.visible = false;
  }
}
}

