/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.business {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;

import mx.core.Application;
import mx.messaging.ChannelSet;
import mx.messaging.channels.AMFChannel;

import mx.messaging.events.MessageEvent;
import mx.messaging.messages.AsyncMessage;

import org.granite.gravity.Consumer;
import org.granite.gravity.channels.GravityChannel;
import org.ow2.orchestra.designer.events.AccessPropertiesEvent;
import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.events.NotificationEvent;
import org.ow2.orchestra.designer.events.WsdlNavigatorEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.NotificationMessage;

/**
 * @author Loic Albertin
 */
public class GravityService {
  private static var _instance:GravityService = null;

  private var consumer:Consumer;

  function GravityService() {
    if (_instance != null) {
      throw new CairngormError(CairngormMessageCodes.SINGLETON_EXCEPTION, "Singleton instance already exists." +
          " Do not call constructor directly, prefer getInstance method");
    }
  }

  public static function getInstance():GravityService {
    if (_instance == null) {
      _instance = new GravityService();
    }
    return _instance;
  }

  public function register():void {
    consumer = new Consumer();
    // Init client-server communications channels properties
    var context:String = Services.parseURL(Application.application.systemManager.loaderInfo.url);

    var amfChannelSet:ChannelSet = new ChannelSet();
    var amfChannel:GravityChannel = new GravityChannel("my-gravityamf", context + "/gravity/amf");

    amfChannelSet.addChannel(amfChannel);
    consumer.channelSet = amfChannelSet;

    consumer.destination = "gravity";
    consumer.topic = DesignerModelLocator.getInstance().userName;

    consumer.addEventListener(MessageEvent.MESSAGE, messageHandlerAck);
    consumer.subscribe();
  }

  private function messageHandlerAck(event:MessageEvent):void {
    var msg:AsyncMessage = event.message as AsyncMessage;

    var subject:String = msg.headers["Subject"];

    new NotificationEvent(NotificationEvent.ADD_NOTIFICATION,
        new NotificationMessage(subject, msg.body as String)).dispatch();

    if (subject == "Access Permissions Updated") {
      // refresh process list
      new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL,
          DesignerModelLocator.getInstance().fileNavigatorModel.selectedItem).dispatch();
      // refresh wsdl list
      new WsdlNavigatorEvent(WsdlNavigatorEvent.FETCH_MODEL).dispatch();
      // refresh list of process with admin rights for the current user
      new AccessPropertiesEvent(AccessPropertiesEvent.FETCH_ADMIN_PROCESSES).dispatch();
    }
  }
}
}
