/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.collections.ArrayCollection;
import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.events.CollectionAddAllEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.utils.ErrorManager;

/**
 * @author Loic Albertin
 */
public class FetchAdminProcessesCMD implements ICommand {
  public function FetchAdminProcessesCMD() {
  }

  public function execute(event:CairngormEvent):void {
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.removeAll();
    var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("processNavigator");
    var token:AsyncToken;
    token = remote.getOwnAdminProcess();
    // add listener for the async response of remote object and change the cursor to busy
    token.addResponder(new Responder(getProcesses, fault));
  }

  private function fault(fault:FaultEvent):void {
    ErrorManager.getInstance().showError("Remote invoke error: " + fault.message);
  }

  private function getProcesses(result:ResultEvent):void {
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.addAll(
            result.result as ArrayCollection);
    //As COLLECTION_CHANGE dispatch an ADD event for each member of the list
    //we were forced to create and dispatch a specific event
    DesignerModelLocator.getInstance().accessProcessSettings.processesWithAdminPermission.dispatchEvent(
            new CollectionAddAllEvent(CollectionAddAllEvent.COLLECTION_CHANGE_ADD_ALL, ArrayCollection(result.result).toArray()));
  }
}
}
