/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands.fileNavigator {
import com.adobe.cairngorm.business.ServiceLocator;
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;

import mx.rpc.AsyncToken;
import mx.rpc.Responder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;
import mx.rpc.remoting.RemoteObject;

import org.ow2.orchestra.designer.events.FileNavigatorEvent;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.FileNavigatorModel;
import org.ow2.orchestra.designer.utils.ErrorManager;
import org.ow2.orchestra.designer.vo.FileNavigatorProject;

/**
 * Create a new project in processNavigator.
 */
public class NewProjectCMD implements ICommand {
  private var project:FileNavigatorProject;

  public function NewProjectCMD() {
  }

  /**
   * @param event {projectName}
   */
  public function execute(event:CairngormEvent):void {
    var fileNavigatorEvent:FileNavigatorEvent = event as FileNavigatorEvent;
    project = fileNavigatorEvent.item as FileNavigatorProject;
    var fileNavigatorViewModel:FileNavigatorModel = DesignerModelLocator.getInstance().fileNavigatorModel;

    if (fileNavigatorViewModel.projectExist(project.name)) {
      ErrorManager.getInstance().showError("This project '" + project.name + "' already exists.");
    }

    else {
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("processNavigator");
      var token:AsyncToken = remote.createNewProject(project);
      token.addResponder(new Responder(result, fault));
    }
  }

  private function fault(event:FaultEvent):void {
    ErrorManager.getInstance().showError("Remote invoke error: " + event.message);
  }

  private function result(event:ResultEvent):void {
    // Refresh navigator and select project
    new FileNavigatorEvent(FileNavigatorEvent.FETCH_MODEL, project).dispatch();
  }
}
}

