/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.model {
import mx.collections.ArrayCollection;

import org.ow2.orchestra.designer.bpmn.model.IDiagramModel;

import org.ow2.orchestra.designer.utils.DesignerUtil;
/**
 * @author Loic Albertin
 */
public class DiagramEditorModel {
   // Elements currently selected
  private var _selectedElements:ArrayCollection = new ArrayCollection();

  // For undo redo
  // Number of undo/redo elements to keep in memory
  private static const UNDO_REDO_QUEUE_LENGTH:int = 10;
  private var _undoState:ArrayCollection = new ArrayCollection();
  private var _redoState:ArrayCollection = new ArrayCollection();

  private var _diagramModel:IDiagramModel;

  public function DiagramEditorModel(diagramModel:IDiagramModel) {
    _diagramModel= diagramModel;
  }

  public function get selectedElements():ArrayCollection {
    return _selectedElements;
  }

  public function set selectedElements(value:ArrayCollection):void {
    _selectedElements = value;
  }

   public function saveModelForUndo():void {
    _undoState.addItem(DesignerUtil.copy(_diagramModel));
    if (_undoState.length > UNDO_REDO_QUEUE_LENGTH) {
      // undo max length is reached, remove the oldest process state
      _undoState.removeItemAt(0);
    }
    _redoState.removeAll();
  }

  public function undo():void {
    trace("Undo..");
    if (_undoState.length != 0) {
      _redoState.addItem(DesignerUtil.copy(_diagramModel));
      diagramModel =  IDiagramModel(_undoState.removeItemAt(_undoState.length - 1));
      trace("Undo done");
    }
  }

  public function redo():void {
    if (_redoState.length != 0) {
      _undoState.addItem(DesignerUtil.copy(_diagramModel));
      diagramModel = IDiagramModel(_redoState.removeItemAt(_redoState.length - 1));
    }
  }

  public function get diagramModel():IDiagramModel {
    return _diagramModel;
  }

  public function set diagramModel(value:IDiagramModel):void {
    _diagramModel = value;
    _selectedElements.removeAll();
  }

}
}
