/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view {
  import flash.events.Event;
  import flash.events.KeyboardEvent;
  import flash.events.MouseEvent;
  import flash.geom.Point;
  import flash.ui.Keyboard;

  import mx.containers.Canvas;
  import mx.containers.HBox;
  import mx.containers.VBox;
  import mx.controls.Button;
  import mx.controls.Image;
  import mx.controls.Label;
  import mx.controls.Spacer;
  import mx.controls.TextInput;
  import mx.effects.Resize;
  import mx.effects.easing.Bounce;
  import mx.events.FlexEvent;
  import mx.events.ResizeEvent;
  import mx.states.AddChild;
  import mx.states.RemoveChild;
  import mx.states.SetEventHandler;
  import mx.states.State;
  import mx.states.Transition;

  import org.ow2.orchestra.designer.bpmn.model.FlowModel;
  import org.ow2.orchestra.designer.bpmn.model.data.AbstractExpressionElementModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataObjectModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionOperationModel;
  import org.ow2.orchestra.designer.bpmn.model.data.LiteralModel;
  import org.ow2.orchestra.designer.bpmn.model.process.EventModel;
  import org.ow2.orchestra.designer.bpmn.model.process.EventType;
  import org.ow2.orchestra.designer.bpmn.model.process.GatewayModel;
  import org.ow2.orchestra.designer.bpmn.model.process.GatewayType;
  import org.ow2.orchestra.designer.bpmn.model.process.TaskModel;
  import org.ow2.orchestra.designer.bpmn.model.process.TaskType;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.FlowView;
  import org.ow2.orchestra.designer.bpmn.views.data.AbstractExpressionElementView;
  import org.ow2.orchestra.designer.bpmn.views.data.DataFlowView;
  import org.ow2.orchestra.designer.bpmn.views.data.ExpressionOperationView;
  import org.ow2.orchestra.designer.bpmn.views.data.LiteralView;
  import org.ow2.orchestra.designer.bpmn.views.process.EventView;
  import org.ow2.orchestra.designer.bpmn.views.process.GatewayView;
  import org.ow2.orchestra.designer.bpmn.views.process.SequenceFlowView;
  import org.ow2.orchestra.designer.bpmn.views.process.TaskView;
  import org.ow2.orchestra.designer.events.DiagramEvent;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
  import org.ow2.orchestra.designer.events.FlowOrderingEvent;
  import org.ow2.orchestra.designer.utils.ContextPanelResources;
  import org.ow2.orchestra.designer.view.dataMappingEditor.DataMappingEditorView;
  import org.ow2.orchestra.designer.view.diagramEditor.AbstractDiagramEditorView;

  public class ElementContextPanel extends Canvas {

    private var removeButton:GlowButton = new GlowButton();


    // embedded palette buttons
    private var linkButton:GlowButton = new GlowButton();
    private var addGatewayButton:GlowButton = new GlowButton();
    private var addEventButton:GlowButton = new GlowButton();
    private var addTaskButton:GlowButton = new GlowButton();

    // embedded palette disabled buttons
    private var linkButtonDisabled:Image = new Image();
    private var addGatewayButtonDisabled:Image = new Image();
    private var addEventButtonDisabled:Image = new Image();
    private var addTaskButtonDisabled:Image = new Image();

    // change gateway type buttons
    private var parallelTypeImage:Image = new GlowButton();
    private var exclusiveTypeImage:Image = new GlowButton();
    private var exclusiveEventTypeImage:Image = new GlowButton();

    // change event type buttons
    private var noneTypeImage:Image = new GlowButton();
    private var sendTypeImage:Image = new GlowButton();
    private var timerTypeImage:Image = new GlowButton();
    private var receiveTypeImage:Image = new GlowButton();

    // change task type buttons
    private var serviceTypeImage:Image = new GlowButton();
    private var userTypeImage:Image = new GlowButton();

    // rename elements
    private var renameButton:GlowButton = new GlowButton();
    private var renameTextInput:TextInput = new TextInput();

    // Flow ordering element
    private var reorderButton:GlowButton = new GlowButton();

    // select literal elements
    private var selectLiteralButton:GlowButton = new GlowButton();

    // context panel boxes
    private var rightBox:VBox = new VBox();
    private var topBox:HBox = new HBox();
    private var topBoxWrapper:HBox = new HBox();
    private var leftBox:VBox = new VBox();

    [Bindable]
    private var _attachedView:AbstractElementView;

    /**
     * Images for the buttons
     */
    private static const btn_remove:Class = ContextPanelResources.getInstance().btn_remove;
    private static const btn_reorder:Class = ContextPanelResources.getInstance().btn_reorder;

    private static const btn_link:Class = ContextPanelResources.getInstance().btn_link;
    private static const btn_link_disabled:Class = ContextPanelResources.getInstance().btn_link_disabled;
    private static const btn_rename:Class = ContextPanelResources.getInstance().btn_label;
    private static const btn_add_event:Class = ContextPanelResources.getInstance().btn_add_event;
    private static const btn_add_gateway:Class = ContextPanelResources.getInstance().btn_add_gateway;
    private static const btn_add_task:Class = ContextPanelResources.getInstance().btn_add_task;
    private static const btn_add_event_disabled:Class = ContextPanelResources.getInstance().btn_add_event_disabled;
    private static const btn_add_gateway_disabled:Class = ContextPanelResources.getInstance().btn_add_gateway_disabled;
    private static const btn_add_task_disabled:Class = ContextPanelResources.getInstance().btn_add_task_disabled;

    /**
     * The source for image data binding into the palette.
     */
    [Embed(source="/../assets/palette/gatewayDefault.svg")]
    private static const defaultImgGateway:Class;
    [Embed(source="/../assets/palette/eventDefault.svg")]
    private static const defaultImgEvent:Class;
    [Embed(source="/../assets/palette/taskDefault.svg")]
    private static const defaultImgTask:Class;

    private var _diagramEditorView:AbstractDiagramEditorView;

    /**
     * Background image
     */
    private static const balloon:Class = ContextPanelResources.getInstance().balloon;

    public function ElementContextPanel(diagramEditorView:AbstractDiagramEditorView) {
      _diagramEditorView = diagramEditorView;
      init();
    }

    public function init():void {
      this.verticalScrollPolicy = "off";
      this.horizontalScrollPolicy = "off";
      this.styleName = "Balloon";

      removeButton.id = "remove";
      removeButton.automationName = "remove";
      renameButton.id = "rename";
      renameButton.automationName = "rename";

      renameButton.source = btn_rename;
      removeButton.source = btn_remove;

      selectLiteralButton.id = "selectLiteral";
      selectLiteralButton.automationName = "selectLiteral";
      selectLiteralButton.source = btn_rename;

      linkButtonDisabled.source = btn_link_disabled;
      linkButtonDisabled.automationName = "createLinkDisabled";
      linkButtonDisabled.width = 15;
      linkButtonDisabled.height = 15;

      addGatewayButtonDisabled.source = btn_add_gateway_disabled;
      addGatewayButtonDisabled.automationName = "addGatewayDisabled";
      addGatewayButtonDisabled.width = 15;
      addGatewayButtonDisabled.height = 15;

      addEventButtonDisabled.automationName = "addEventDisabled";
      addEventButtonDisabled.source = btn_add_event_disabled;
      addEventButtonDisabled.width = 15;
      addEventButtonDisabled.height = 15;

      addTaskButtonDisabled.automationName = "addTaskDisabled";
      addTaskButtonDisabled.source = btn_add_task_disabled;
      addTaskButtonDisabled.width = 15;
      addTaskButtonDisabled.height = 10;

      if (_diagramEditorView is DataMappingEditorView) {
        linkButton.id = "createLinkDataMapping";
        linkButton.automationName = "createLinkDataMapping";
      } else {
        linkButton.id = "createLink";
        linkButton.automationName = "createLink";
      }
      linkButton.source = btn_link;
      linkButton.width = 15;
      linkButton.height = 15;
      linkButton.addEventListener(MouseEvent.CLICK, handleLinkClick);

      addGatewayButton.id = "addGateway";
      addGatewayButton.automationName = "addGateway";
      addGatewayButton.source = btn_add_gateway;
      addGatewayButton.width = 15;
      addGatewayButton.height = 15;
      addGatewayButton.addEventListener(MouseEvent.CLICK, handleAddGatewayClick);

      addEventButton.id = "addEvent";
      addEventButton.automationName = "addEvent";
      addEventButton.source = btn_add_event;
      addEventButton.width = 15;
      addEventButton.height = 15;
      addEventButton.addEventListener(MouseEvent.CLICK, handleAddEventClick);

      addTaskButton.id = "addTask";
      addTaskButton.automationName = "addTask";
      addTaskButton.source = btn_add_task;
      addTaskButton.width = 15;
      addTaskButton.height = 10;
      addTaskButton.addEventListener(MouseEvent.CLICK, handleAddTaskClick);


      removeButton.addEventListener(MouseEvent.CLICK, handleRemoveClick);
      renameButton.addEventListener(MouseEvent.CLICK, handleRenameClick);
      selectLiteralButton.addEventListener(MouseEvent.CLICK, handleSelectLiteralClick);

      parallelTypeImage.source = ContextPanelResources.getInstance().gateway_parallel_type;
      parallelTypeImage.width = 15;
      parallelTypeImage.height = 15;
      parallelTypeImage.data = GatewayType.PARALLEL as Object;
      parallelTypeImage.toolTip = "parallel";
      parallelTypeImage.automationName = "parallelGateway";

      exclusiveTypeImage.source = ContextPanelResources.getInstance().gateway_exclusive_type;
      exclusiveTypeImage.width = 15;
      exclusiveTypeImage.height = 15;
      exclusiveTypeImage.data = GatewayType.EXCLUSIVE as Object;
      exclusiveTypeImage.toolTip = "exclusive";
      exclusiveTypeImage.automationName = "exclusiveGateway";

      exclusiveEventTypeImage.source = ContextPanelResources.getInstance().gateway_exclusive_event_type;
      exclusiveEventTypeImage.width = 15;
      exclusiveEventTypeImage.height = 15;
      exclusiveEventTypeImage.data = GatewayType.EXCLUSIVE_EVENT as Object;
      exclusiveEventTypeImage.toolTip = "exclusive event";
      exclusiveEventTypeImage.automationName = "exclusiveEventGateway";

      parallelTypeImage.addEventListener(MouseEvent.CLICK, handleChangeGatewayTypeClick);
      exclusiveTypeImage.addEventListener(MouseEvent.CLICK, handleChangeGatewayTypeClick);
      exclusiveEventTypeImage.addEventListener(MouseEvent.CLICK, handleChangeGatewayTypeClick);


      noneTypeImage.source = ContextPanelResources.getInstance().btn_add_event;
      noneTypeImage.width = 15;
      noneTypeImage.height = 15;
      noneTypeImage.data = EventType.NONE as Object;
      noneTypeImage.toolTip = "none";
      noneTypeImage.automationName = "noneEvent";

      receiveTypeImage.source = ContextPanelResources.getInstance().event_receive_message_type;
      receiveTypeImage.width = 15;
      receiveTypeImage.height = 15;
      receiveTypeImage.data = EventType.RECEIVE_MESSAGE as Object;
      receiveTypeImage.toolTip = "receive";
      receiveTypeImage.automationName = "receiveEvent";

      sendTypeImage.source = ContextPanelResources.getInstance().event_send_message_type;
      sendTypeImage.width = 15;
      sendTypeImage.height = 15;
      sendTypeImage.data = EventType.SEND_MESSAGE as Object;
      sendTypeImage.toolTip = "send";
      sendTypeImage.automationName = "sendEvent";

      timerTypeImage.source = ContextPanelResources.getInstance().event_timer_type;
      timerTypeImage.width = 15;
      timerTypeImage.height = 15;
      timerTypeImage.data = EventType.TIMER as Object;
      timerTypeImage.toolTip = "timer";
      timerTypeImage.automationName = "timerEvent";

      noneTypeImage.addEventListener(MouseEvent.CLICK, handleChangeEventTypeClick);
      receiveTypeImage.addEventListener(MouseEvent.CLICK, handleChangeEventTypeClick);
      sendTypeImage.addEventListener(MouseEvent.CLICK, handleChangeEventTypeClick);
      timerTypeImage.addEventListener(MouseEvent.CLICK, handleChangeEventTypeClick);

      serviceTypeImage.source = ContextPanelResources.getInstance().task_service_type;
      serviceTypeImage.width = 15;
      serviceTypeImage.height = 15;
      serviceTypeImage.data = TaskType.SERVICE as Object;
      serviceTypeImage.toolTip = "service";
      serviceTypeImage.automationName = "serviceTask";

      userTypeImage.source = ContextPanelResources.getInstance().task_user_type;
      userTypeImage.width = 15;
      userTypeImage.height = 15;
      userTypeImage.data = TaskType.USER as Object;
      userTypeImage.toolTip = "user";
      userTypeImage.automationName = "userTask";

      serviceTypeImage.addEventListener(MouseEvent.CLICK, handleChangeTaskTypeClick);
      userTypeImage.addEventListener(MouseEvent.CLICK, handleChangeTaskTypeClick);

      reorderButton.id = "reorder";
      reorderButton.automationName = "reorder";
      reorderButton.source = btn_reorder;
      reorderButton.width = 15;
      reorderButton.height = 15;
      reorderButton.addEventListener(MouseEvent.CLICK, handleReorderClick);

      rightBox.setStyle("verticalGap", 2);
      rightBox.setStyle("horizontalGap", 2);
      topBox.setStyle("verticalGap", 2);
      topBox.setStyle("horizontalGap", 2);
      leftBox.setStyle("verticalGap", 2);
      leftBox.setStyle("horizontalGap", 2);

      // wrap topBox in another HBox that contains spacers
//    var topBoxWrapper:HBox = new HBox();
//    topBoxWrapper.setStyle("verticalGap", 2);
//    topBoxWrapper.setStyle("horizontalGap", 2);
      topBoxWrapper.percentWidth = 100;

      var leftSpacer:Spacer = new Spacer();
      leftSpacer.percentWidth = 100;
      leftSpacer.setStyle("backgroundColor", "red");

      topBoxWrapper.addChild(leftSpacer);
      topBoxWrapper.addChild(topBox);
      var rightSpacer:Spacer = new Spacer();
      rightSpacer.percentWidth = 100;
      rightSpacer.setStyle("backgroundColor", "green");
      topBoxWrapper.addChild(rightSpacer);

      this.addChild(topBoxWrapper);
      this.addChild(leftBox);
      this.addChild(rightBox);

      renameTextInput.percentWidth = 100;
      renameTextInput.automationName = "renameTextInput";


      // Add states to the view
      var defaultState:State = new State();
      defaultState.name = "default";

      this.states.push(defaultState, getRemoveState(), getRenameState());
      this.currentState = "default";

      // Add transitions between states
      // create resize effect
      var resize:Resize = new Resize(this);
      resize.easingFunction = Bounce.easeOut;


      var defaultToDelete:Transition = new Transition();
      defaultToDelete.effect = resize;
      defaultToDelete.fromState = "default";
      defaultToDelete.toState = "delete";

      transitions.push(defaultToDelete);

      this.addEventListener(ResizeEvent.RESIZE, resetPosition);
      topBox.addEventListener(ResizeEvent.RESIZE, resetPosition);
      leftBox.addEventListener(ResizeEvent.RESIZE, resetPosition);
      rightBox.addEventListener(ResizeEvent.RESIZE, resetPosition);
    }


    private function getRemoveState():State {
      var state:State = new State();
      state.name = "remove";

      this.addRemoveButtonChildrenToState(state);

      var hbox1:HBox = new HBox();
      hbox1.percentWidth = 100;
      var spacer1:Spacer = new Spacer();
      spacer1.percentWidth = 100;
      var spacer2:Spacer = new Spacer();
      spacer2.percentWidth = 100;
      var label:Label = new Label();
      label.text = "Remove me?";

      hbox1.addChild(spacer1);
      hbox1.addChild(label);
      hbox1.addChild(spacer2);

      var hbox2:HBox = new HBox();
      hbox2.percentWidth = 100;
      var spacer3:Spacer = new Spacer();
      spacer3.percentWidth = 100;
      var spacer4:Spacer = new Spacer();
      spacer4.percentWidth = 100;
      var button0:Button = new Button();
      button0.width = 50;
      button0.label = "Yes";
      button0.automationName = "yes";
      button0.addEventListener(MouseEvent.CLICK, handleRemoveYesClick);
      var button1:Button = new Button();
      button1.label = "No";
      button1.width = 50;
      button1.automationName = "no";
      button1.addEventListener(MouseEvent.CLICK, handleRemoveNoClick);

      hbox2.addChild(spacer3);
      hbox2.addChild(button0);
      hbox2.addChild(button1);
      hbox2.addChild(spacer4);

      var vbox:VBox = new VBox();
      vbox.addChild(hbox1);
      vbox.addChild(hbox2);
      vbox.setStyle("borderSkin", balloon);
      vbox.setStyle("paddingBottom", 10);
      vbox.setStyle("paddingLeft", 10);
      vbox.setStyle("paddingRight", 10);
      vbox.setStyle("paddingTop", 10);
      state.overrides.push(new AddChild(this, vbox));

      return state;
    }


    private function handleRemoveClick(event:MouseEvent):void {
      this.currentState = "remove";
    }


    private function handleRemoveYesClick(event:MouseEvent):void {

      new DiagramEvent(
              DiagramEvent.SELECTED_DIAGRAM_ELEMENTS_DELETE,
              _diagramEditorView.diagramEditorModel).dispatch();
      this.currentState = "default";
    }

    private function handleRemoveNoClick(event:MouseEvent):void {
      this.currentState = "default";
    }

    private function getRenameState():State {
      var state:State = new State();
      state.name = "rename";

      this.addRemoveButtonChildrenToState(state);

      var hbox1:HBox = new HBox();
      hbox1.percentWidth = 100;
      var spacer1:Spacer = new Spacer();
      spacer1.percentWidth = 100;
      var spacer2:Spacer = new Spacer();
      spacer2.percentWidth = 100;
      var label:Label = new Label();
      label.text = "Element name ?";

      hbox1.addChild(spacer1);
      hbox1.addChild(label);
      hbox1.addChild(spacer2);

      var hbox2:HBox = new HBox();
      hbox2.percentWidth = 100;
      var spacer3:Spacer = new Spacer();
      spacer3.percentWidth = 100;
      var spacer4:Spacer = new Spacer();
      spacer4.percentWidth = 100;
      var button0:Button = new Button();
      button0.width = 70;
      button0.label = "Ok";
      button0.automationName = "ok";
      button0.addEventListener(MouseEvent.CLICK, handleRenameOkClick);
      var button1:Button = new Button();
      button1.label = "Cancel";
      button1.width = 70;
      button1.automationName = "cancel";
      button1.addEventListener(MouseEvent.CLICK, handleRenameCancelClick);

      hbox2.addChild(spacer3);
      hbox2.addChild(button0);
      hbox2.addChild(button1);
      hbox2.addChild(spacer4);

      var vbox:VBox = new VBox();
      vbox.addChild(hbox1);
      vbox.addChild(renameTextInput);
      vbox.addChild(hbox2);
      vbox.setStyle("borderSkin", balloon);
      vbox.setStyle("paddingBottom", 10);
      vbox.setStyle("paddingLeft", 10);
      vbox.setStyle("paddingRight", 10);
      vbox.setStyle("paddingTop", 10);
      state.overrides.push(new AddChild(this, vbox));

      var keyboardEventHandler:SetEventHandler = new SetEventHandler(this, KeyboardEvent.KEY_DOWN);
      keyboardEventHandler.handlerFunction = handleRenameKeyPress;
      state.overrides.push(keyboardEventHandler);

      return state;
    }

    private function handleRenameKeyPress(event:KeyboardEvent):void {
      if (event.keyCode == Keyboard.ENTER) {
        handleRenameOkClick();
      } else if (event.keyCode == Keyboard.ESCAPE) {
        handleRenameCancelClick();
      }
    }

    private function handleRenameCancelClick(event:MouseEvent = null):void {
      // reset view state
      this.currentState = "default";
    }

    private function handleRenameOkClick(event:MouseEvent = null):void {
      // commit changes to model name.
      var evt:ElementPropertyChangeEvent =
              new ElementPropertyChangeEvent(
                      ElementPropertyChangeEvent.PROPERTY_CHANGED, _diagramEditorView.diagramEditorModel,
                      attachedView.model,
                      "name",
                      renameTextInput.text
              );
      evt.dispatch();
      // reset view state
      this.currentState = "default";
    }

    private function handleRenameClick(event:MouseEvent):void {
      // Switch view to rename state
      this.currentState = "rename";
      // prepare text input
      renameTextInput.text = attachedView.model.name;
      renameTextInput.selectionBeginIndex = renameTextInput.text.length;
      renameTextInput.selectionEndIndex = renameTextInput.text.length;
      // set focus to text input
      renameTextInput.setFocus();
    }

    private function handleReorderClick(event:MouseEvent):void {
      // commit changes to model name.
      var evt:FlowOrderingEvent =
              new FlowOrderingEvent(FlowModel(attachedView.model), _diagramEditorView.diagramEditorModel);
      evt.dispatch();
      // reset view state
      this.currentState = "default";
    }

    public function handleSelectLiteralClick(event:MouseEvent):void {
      if (this._diagramEditorView is DataMappingEditorView) {
        DataMappingEditorView(_diagramEditorView).showLiteralSelectorPanel();
      }
    }

    private function handleLinkClick(event:MouseEvent):void {
      // switch to creating flow state
      var diagramEditorView:AbstractDiagramEditorView = this.parent as AbstractDiagramEditorView;
      diagramEditorView.currentState = "creatingFlow";
      event.stopImmediatePropagation();
    }

    private function handleAddGatewayClick(event:MouseEvent):void {
      // switch to creating flow state
      var diagramEditorView:AbstractDiagramEditorView = this.parent as AbstractDiagramEditorView;
      diagramEditorView.imageOfElementToAdd.source = defaultImgGateway;
      diagramEditorView.modelOfElementToAdd = new GatewayModel();
      diagramEditorView.currentState = "creatingElement";
      event.stopImmediatePropagation();
    }

    private function handleAddEventClick(event:MouseEvent):void {
      // switch to creating flow state
      var diagramEditorView:AbstractDiagramEditorView = this.parent as AbstractDiagramEditorView;
      diagramEditorView.imageOfElementToAdd.source = defaultImgEvent;
      diagramEditorView.modelOfElementToAdd = new EventModel();
      diagramEditorView.modelOfElementToAdd.name = "event";
      diagramEditorView.currentState = "creatingElement";
      event.stopImmediatePropagation();
    }

    private function handleAddTaskClick(event:MouseEvent):void {
      // switch to creating flow state
      var diagramEditorView:AbstractDiagramEditorView = this.parent as AbstractDiagramEditorView;
      diagramEditorView.imageOfElementToAdd.source = defaultImgTask;
      diagramEditorView.modelOfElementToAdd = new TaskModel();
      diagramEditorView.modelOfElementToAdd.name = "task";
      diagramEditorView.currentState = "creatingElement";
      event.stopImmediatePropagation();
    }

    private function addChangeEventTypeButtonsToLeftBox():void {
      leftBox.addChild(noneTypeImage);
      leftBox.addChild(receiveTypeImage);
      leftBox.addChild(sendTypeImage);
      leftBox.addChild(timerTypeImage);
    }

    private function addChangeGatewayTypeButtonsToLeftBox():void {
      leftBox.addChild(parallelTypeImage);
      leftBox.addChild(exclusiveTypeImage);
      leftBox.addChild(exclusiveEventTypeImage);
    }

    private function addChangeTaskTypeButtonsToLeftBox():void {
      leftBox.addChild(serviceTypeImage);
      leftBox.addChild(userTypeImage);
    }

    private function handleChangeEventTypeClick(event:MouseEvent):void {
      // commit changes to model type.
      var evt:ElementPropertyChangeEvent =
              new ElementPropertyChangeEvent(
                      ElementPropertyChangeEvent.PROPERTY_CHANGED, _diagramEditorView.diagramEditorModel,
                      attachedView.model,
                      "eventType",
                      event.currentTarget.data
              );
      evt.dispatch();
      // reset view state
      this.currentState = "default";
    }

    private function handleChangeGatewayTypeClick(event:MouseEvent):void {
      // commit changes to model type.
      var evt:ElementPropertyChangeEvent =
              new ElementPropertyChangeEvent(
                      ElementPropertyChangeEvent.PROPERTY_CHANGED, _diagramEditorView.diagramEditorModel,
                      attachedView.model,
                      "gatewayType",
                      event.currentTarget.data
              );
      evt.dispatch();
      // reset view state
      this.currentState = "default";
    }

    private function handleChangeTaskTypeClick(event:MouseEvent):void {
      // commit changes to model type.
      var evt:ElementPropertyChangeEvent =
              new ElementPropertyChangeEvent(
                      ElementPropertyChangeEvent.PROPERTY_CHANGED, _diagramEditorView.diagramEditorModel,
                      attachedView.model,
                      "taskType",
                      event.currentTarget.data
              );
      evt.dispatch();
      // reset view state
      this.currentState = "default";
    }

    /**
     * Method to set-up a contextPanel state to hide the contextPanel buttons.
     * @param state
     */
    private function addRemoveButtonChildrenToState(state:State):void {
      state.overrides.push(new RemoveChild(this.topBox));
      state.overrides.push(new RemoveChild(this.leftBox));
      state.overrides.push(new RemoveChild(this.rightBox));
    }

    public function get attachedView():AbstractElementView {
      return _attachedView;
    }

    public function set attachedView(value:AbstractElementView):void {
      if (attachedView != null) {
        attachedView.removeEventListener(FlexEvent.CREATION_COMPLETE, resetPosition);
        attachedView.removeEventListener("widthChanged", resetPosition);
        attachedView.removeEventListener("heightChanged", resetPosition);
        attachedView.removeEventListener("xChanged", resetPosition);
        attachedView.removeEventListener("yChanged", resetPosition);
      }
      _attachedView = value;

      rightBox.removeAllChildren();
      topBox.removeAllChildren();
      leftBox.removeAllChildren();

      if (attachedView is GatewayView) {
        topBox.addChild(removeButton);
        addChangeGatewayTypeButtonsToLeftBox();
        addNewElementsButtonsToRightBox();
      } else if (attachedView is SequenceFlowView) {
        topBox.addChild(removeButton);
      } else if (attachedView is EventView) {
        topBox.addChild(renameButton);
        topBox.addChild(removeButton);

        addChangeEventTypeButtonsToLeftBox();

        if ((attachedView.model as EventModel).outgoingFlows.length < 1) {
          addNewElementsButtonsToRightBox();
        } else {
          addNewElementsDisabledButtonsToRightBox();
        }
      } else if (attachedView is TaskView) {
        topBox.addChild(renameButton);
        topBox.addChild(removeButton);
        addChangeTaskTypeButtonsToLeftBox();

        if ((attachedView.model as TaskModel).outgoingFlows.length < 1) {
          addNewElementsButtonsToRightBox();
        } else {
          addTaskNewElementsDisabledButtonsToRightBox();
        }
      } else if (attachedView is AbstractExpressionElementView) {
        topBox.addChild(removeButton);
        var dataModel:AbstractExpressionElementModel = (attachedView.model as AbstractExpressionElementModel);
        if (dataModel is ExpressionDataObjectModel) {
          if (dataModel.outgoingFlows.length < 1) {
            addNewLinkButtonsToRightBox();
          } else {
            addNewLinkDisabledButtonsToRightBox();
          }
        }
      } else if (attachedView is LiteralView) {
        topBox.addChild(selectLiteralButton);
        topBox.addChild(removeButton);
        var literalModel:LiteralModel = (attachedView.model as LiteralModel);
        if (literalModel.outgoingFlows.length < 1) {
          addNewLinkButtonsToRightBox();
        } else {
          addNewLinkDisabledButtonsToRightBox();
        }
      } else if (attachedView is ExpressionOperationView) {
        topBox.addChild(removeButton);
        var expressionOperationModel:ExpressionOperationModel = (attachedView.model as ExpressionOperationModel);
        if (expressionOperationModel.outgoingFlows.length < 1) {
          addNewLinkButtonsToRightBox();
        } else {
          addNewLinkDisabledButtonsToRightBox();
        }
      } else if (attachedView is DataFlowView) {
        if (FlowModel(attachedView.model).targetElement is ExpressionOperationModel) {
          topBox.addChild(reorderButton);
        }
        topBox.addChild(removeButton);
      }

      resetPosition();
      attachedView.addEventListener(FlexEvent.CREATION_COMPLETE, resetPosition);
      attachedView.addEventListener("widthChanged", resetPosition);
      attachedView.addEventListener("heightChanged", resetPosition);
      attachedView.addEventListener("xChanged", resetPosition);
      attachedView.addEventListener("yChanged", resetPosition);
    }


    private function addNewElementsButtonsToRightBox():void {
      rightBox.addChild(linkButton);
      rightBox.addChild(addGatewayButton);
      rightBox.addChild(addEventButton);
      rightBox.addChild(addTaskButton);
    }

    private function addNewElementsDisabledButtonsToRightBox():void {
      rightBox.addChild(linkButtonDisabled);
      rightBox.addChild(addGatewayButtonDisabled);
      rightBox.addChild(addEventButtonDisabled);
      rightBox.addChild(addTaskButtonDisabled);
    }

    private function addNewLinkButtonsToRightBox():void {
      rightBox.addChild(linkButton);
    }

    private function addNewLinkDisabledButtonsToRightBox():void {
      rightBox.addChild(linkButtonDisabled);
    }

    private function addTaskNewElementsDisabledButtonsToRightBox():void {
      rightBox.addChild(linkButton);
      rightBox.addChild(addGatewayButtonDisabled);
      rightBox.addChild(addEventButtonDisabled);
      rightBox.addChild(addTaskButtonDisabled);
    }

    /**
     * Update panel position.
     * Called when the content is changed (on resize event).
     * @param event
     */
    private function resetPosition(event:Event = null):void {
      if (attachedView != null) {
        // set context panel position
        var localPoint:Point;
        if (attachedView is FlowView) {
          var flowView:FlowView = attachedView as FlowView;
          localPoint = new Point(
                  (flowView.sourcePoint.x + flowView.targetPoint.x) / 2,
                  (flowView.sourcePoint.y + flowView.targetPoint.y - this.height) / 2);
        } else {
          localPoint = new Point(
                  attachedView.x - leftBox.width - 5,
                  attachedView.y - topBoxWrapper.height - 5);
          leftBox.x = 0;
          leftBox.y = topBoxWrapper.height + 5;
          rightBox.x = attachedView.width + leftBox.width + 10;
          rightBox.y = topBoxWrapper.height + 5;
        }
        this.x = localPoint.x - _diagramEditorView.diagramView.horizontalScrollPosition;
        this.y = localPoint.y - _diagramEditorView.diagramView.verticalScrollPosition;
      }
    }


  }
}
