/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

/**
 * @author Jerome Camilleri
 */
package org.ow2.orchestra.designer.view.dataMappingEditor {
import mx.binding.utils.BindingUtils;
import mx.containers.VBox;
import mx.controls.Label;
import mx.events.CollectionEvent;
import mx.events.FlexEvent;
import mx.states.AddChild;
import mx.states.State;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.DataMappingModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventType;
import org.ow2.orchestra.designer.bpmn.model.process.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.process.GatewayType;
import org.ow2.orchestra.designer.bpmn.model.process.SequenceFlowModel;
import org.ow2.orchestra.designer.bpmn.model.process.TaskModel;
import org.ow2.orchestra.designer.model.DataMappingEditorModel;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.view.palette.datamapping.DataMappingPalette;

/**
 * The view for tab dataMapping
 */
public class DataMapperView extends VBox {
  /**
   * The source for "properties" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/icon/container/dataMapping.gif")]
  public var dataMapping:Class;

  private var processEditorModel:ProcessEditorModel;
  private var _dataMappingEditorView:DataMappingEditorView;

  public function DataMapperView() {
    super();

    this.percentHeight = 100;
    this.percentWidth = 100;
    this.label = "Data Mapping";
    this.icon = dataMapping;

    var enabledState:State = new State();
    enabledState.name = "enabled";
    enabledState.overrides.push(new AddChild(this, new DataMappingPalette()));
//    enabledState.overrides.push(new AddChild(this, new DataMappingEditorView(new DataMappingEditorModel())));
    enabledState.addEventListener(FlexEvent.ENTER_STATE, enabledStateEnter);
    enabledState.addEventListener(FlexEvent.EXIT_STATE, enabledStateExit);
    this.states.push(enabledState);

    var disabledState:State = new State();
    disabledState.name = "disabled";
    var disabledLabel:Label = new Label();
    disabledLabel.text = "Data mapping not available for the selected element.";
    disabledLabel.percentHeight = 100;
    disabledLabel.percentWidth = 100;
    disabledState.overrides.push(new AddChild(this, disabledLabel));
    this.states.push(disabledState);

    // Active the tab matching to the selected process
    BindingUtils.bindSetter(setSelectedProcessEditor, DesignerModelLocator.getInstance(), "selectedProcessEditor");

    // set empty properties view
    selectedElementChanged();

  }

  private function enabledStateExit(event:FlexEvent):void {
    if (_dataMappingEditorView != null) {
      this.removeChild(_dataMappingEditorView);
      _dataMappingEditorView.dispose();
    }
    _dataMappingEditorView = null;
  }

  private function enabledStateEnter(event:FlexEvent):void {
    var dataMappingModel:DataMappingModel = null;
    var selectedElement:AbstractElement = processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
    if (selectedElement is EventModel) {
      if (EventModel(selectedElement).dataMappingModel == null) {
        EventModel(selectedElement).dataMappingModel = new DataMappingModel();
      }
      dataMappingModel = EventModel(selectedElement).dataMappingModel;
    } else if (selectedElement is TaskModel) {
      if (TaskModel(selectedElement).dataMappingModel == null) {
        TaskModel(selectedElement).dataMappingModel = new DataMappingModel();
      }
      dataMappingModel = TaskModel(selectedElement).dataMappingModel;
    } else if (selectedElement is SequenceFlowModel) {
      if (SequenceFlowModel(selectedElement).dataMappingModel == null) {
        SequenceFlowModel(selectedElement).dataMappingModel = new DataMappingModel();
      }
      dataMappingModel = SequenceFlowModel(selectedElement).dataMappingModel;
    }
    _dataMappingEditorView = new DataMappingEditorView(new DataMappingEditorModel(dataMappingModel));
    this.addChild(_dataMappingEditorView);
  }

  /**
   * Active the tab matching to the selected process
   */
  private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
    if (processEditorModel != null) {
      processEditorModel.selectedElements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
    }
    processEditorModel = selectedProcessEditor;
    if (selectedProcessEditor != null) {
      selectedProcessEditor.selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
    }
    selectedElementChanged();
  }

  /**
   * When the collection is modified then we update the dataMapper
   * Note: when a element of the collection is updated (one of its values changed) the a collection event
   * {@link mx.events.CollectionEventKind@UPDATE} is dispatched. So we do not have to listen on properties changed events for
   * selected
   * elements.
   * @param event A collection event
   */
  private function selectedElementChanged(event:CollectionEvent = null):void {
    if (processEditorModel != null) {
      if (processEditorModel.selectedElements.length == 1) {
        var selectedElement:AbstractElement = processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
        if ((selectedElement is EventModel && (selectedElement as EventModel).eventType == EventType.SEND_MESSAGE )
                    || selectedElement is TaskModel
                || (selectedElement is SequenceFlowModel && SequenceFlowModel(selectedElement).sourceElement is GatewayModel
                && GatewayModel(SequenceFlowModel(selectedElement).sourceElement).gatewayType == GatewayType.EXCLUSIVE)) {
          this.currentState = "enabled";
        } else {
          this.currentState = "disabled";
        }
      } else if (processEditorModel.selectedElements.length == 0) {
        this.currentState = "disabled";
      }
    } else {
      this.currentState = "disabled";
    }
  }

}
}
