/**
 * Created by ${PRODUCT_NAME}.
 * User: camillej
 * Date: 24/02/11
 * Time: 13:59
 * To change this template use File | Settings | File Templates.
 */
package org.ow2.orchestra.designer.view.dataMappingEditor {
  import flash.events.MouseEvent;

  import mx.collections.ArrayCollection;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataInputModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataObjectModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionOperationModel;
  import org.ow2.orchestra.designer.bpmn.model.data.LiteralModel;
  import org.ow2.orchestra.designer.view.diagramEditor.AbstractDiagramEditorView;
  import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorFlowStateUtil;

  public class DataMappingEditorFlowStateUtil extends DiagramEditorFlowStateUtil {

    public function DataMappingEditorFlowStateUtil(diagramEditorView:AbstractDiagramEditorView) {
      super(diagramEditorView);
    }

    /**
     *
     * <p>Checks if elements under mouse are allowed to be target of a sequence flow.</p>
     * <p>This implementation check if there is one and only one element under mouse and if this element is not also the source of the sequence flow.</p>
     *
     * @param mouseEvent MouseEvent for moving event
     * @return true if element under mouse is allowed to be source of a sequence flow.
     */
    public override function allowFlowTarget(mouseEvent:MouseEvent):Boolean {
      if (super.allowFlowTarget(mouseEvent)) {
        var listSelectedElements:ArrayCollection = new ArrayCollection();
        listSelectedElements = getElementUnderMouse(mouseEvent);

        if (listSelectedElements.length != 1) {
          return false;
        }
        var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
        if (selectedElement is ExpressionDataInputModel) {
          //Allowing only one incoming flow on ExpressionDataInput
          return ExpressionDataInputModel(selectedElement).incomingFlows.length == 0;
        } else if (selectedElement is ExpressionOperationModel) {
          //Allowing 2 incoming flows on ExpressionDataInput
          return ExpressionOperationModel(selectedElement).incomingFlows.length < 2;
        }
      }
      return false;
    }

    /**
     *
     * <p>Checks if elements under mouse are allowed to be source of a sequence flow.</p>
     * <p>This implementation check if there is one and only one element under mouse and if this element is not also the target of the sequence flow.</p>
     *
     * @param mouseEvent MouseEvent for moving event
     * @return true if element under mouse is allowed to be source of a sequence flow.
     */
    public override function allowFlowSource(mouseEvent:MouseEvent):Boolean {
      if (super.allowFlowSource(mouseEvent)) {
        var listSelectedElements:ArrayCollection = new ArrayCollection();
        listSelectedElements = getElementUnderMouse(mouseEvent);
        if (listSelectedElements.length != 1) {
          return false;
        }
        // only one selected element.
        var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
        if (selectedElement is ExpressionDataObjectModel) {
          //Allowing only one outgoing flow on ExpressionDataObjectModel
          return ExpressionDataObjectModel(selectedElement).outgoingFlows.length == 0;
        } else if (selectedElement is LiteralModel) {
          //Allowing only one incoming flow on LiteralModel
          return LiteralModel(selectedElement).incomingFlows.length == 0;
        } else if (selectedElement is ExpressionOperationModel) {
          //Allowing only one incoming flow on operation model
          return ExpressionOperationModel(selectedElement).incomingFlows.length == 0;
        }
      }
      return false;
    }

  }
}
