/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.view.dataMappingEditor {
  import flash.events.Event;
  import flash.geom.Point;
  import flash.geom.Rectangle;

  import mx.binding.utils.ChangeWatcher;
  import mx.collections.ArrayCollection;
  import mx.collections.ListCollectionView;
  import mx.core.Application;
  import mx.core.ScrollPolicy;
  import mx.events.CollectionEvent;
  import mx.events.CollectionEventKind;
  import mx.events.DragEvent;
  import mx.events.MenuEvent;
  import mx.managers.DragManager;

  import org.granite.collections.BasicMap;
  import org.granite.collections.IMap;
  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.data.AbstractExpressionElementModel;
  import org.ow2.orchestra.designer.bpmn.model.data.DataInputModel;
  import org.ow2.orchestra.designer.bpmn.model.data.DataMappingModel;
  import org.ow2.orchestra.designer.bpmn.model.data.DataObjectModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataInputModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataObjectModel;
  import org.ow2.orchestra.designer.bpmn.model.data.ExpressionOperationModel;
  import org.ow2.orchestra.designer.bpmn.model.data.LiteralModel;
  import org.ow2.orchestra.designer.bpmn.model.data.OperationType;
  import org.ow2.orchestra.designer.bpmn.model.data.type.DataTypeModel;
  import org.ow2.orchestra.designer.bpmn.model.data.type.XSDElementModel;
  import org.ow2.orchestra.designer.bpmn.model.data.type.XSDSequenceType;
  import org.ow2.orchestra.designer.bpmn.model.process.IBPMNElementWithData;
  import org.ow2.orchestra.designer.bpmn.model.utils.DataUtil;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.IFlowNodeView;
  import org.ow2.orchestra.designer.bpmn.views.data.DataMappingView;
  import org.ow2.orchestra.designer.bpmn.views.data.ExpressionOperationView;
  import org.ow2.orchestra.designer.bpmn.views.data.LiteralView;
  import org.ow2.orchestra.designer.events.DiagramModificationEvent;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
  import org.ow2.orchestra.designer.model.DataMappingEditorModel;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.ElementPaletteDnDModel;
  import org.ow2.orchestra.designer.view.PatchedScrollableArrowMenu;
  import org.ow2.orchestra.designer.view.diagramEditor.AbstractDiagramEditorView;
  import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorFlowStateUtil;
  import org.ow2.orchestra.designer.view.menuBar.MenuBarItem;
  import org.ow2.orchestra.designer.view.menuBar.MenuBarItemWithChildren;
  import org.ow2.orchestra.designer.view.palette.AbstractElementPalette;
  import org.ow2.orchestra.designer.view.palette.datamapping.DataInputPaletteView;
  import org.ow2.orchestra.designer.view.palette.datamapping.DataMappingPalette;
  import org.ow2.orchestra.designer.view.palette.datamapping.DataObjectPaletteView;
  import org.ow2.orchestra.designer.view.palette.datamapping.ExpressionOperationPaletteView;
  import org.ow2.orchestra.designer.view.palette.datamapping.LiteralPaletteView;

  /**
   * @author Loic Albertin
   */
  public class DataMappingEditorView extends AbstractDiagramEditorView {
    private var literalSelectorPanel:LiteralSelectorPanel = new LiteralSelectorPanel();

    public function DataMappingEditorView(dataMappingEditorModel:DataMappingEditorModel) {
      super(dataMappingEditorModel);
      elementContextPanel.id = "elementContextPanelDataMapping";
      this.percentHeight = 100;
      this.percentWidth = 100;
      this.automationName = "dataMappingEditorView";
      this.addChild(literalSelectorPanel);
      literalSelectorPanel.visible = false;
      onDataMappingModelChange();
      ChangeWatcher.watch(dataMappingEditorModel, "dataMappingModel", onDataMappingModelChange);

    }

    override protected function createDiagramEditorFlowStateUtil():DiagramEditorFlowStateUtil {
      return new DataMappingEditorFlowStateUtil(this);
    }

    private function onDataMappingModelChange(event:Event = null):void {
      if (_diagramView != null) {
        this.removeChild(_diagramView);
        DataMappingModel(_diagramView.flowModel).elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
      }

      dataMappingEditorModel.dataMappingModel.elements.addEventListener(
              CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);

      _diagramView = new DataMappingView(dataMappingEditorModel.dataMappingModel);
      _diagramView.percentHeight = 100;
      _diagramView.percentWidth = 100;
      this.addChildAt(_diagramView, 0);
    }


    protected override function diagramViewDragEnter(event:DragEvent):void {
      this.dragEnterValid = false;
      this.acceptDrop = false;
      // Accept object
      if (event.dragInitiator is AbstractElementPalette &&
          AbstractElementPalette(event.dragInitiator).parentPalette is DataMappingPalette) {
        DragManager.acceptDragDrop(this);
        DragManager.showFeedback(DragManager.COPY);
        this.acceptDrop = true;
        this.dragEnterValid = true;
      }
    }

    /** Object drag handler */
    protected override function diagramViewDragOver(event:DragEvent):void {
      if (dragEnterValid) {
        if (event.dragInitiator is AbstractElementPalette) {
          var elementRectangle:Rectangle;
          var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
          elementRectangle = elemDnDModel.image.getRect(this);
          if (elementRectangle.topLeft.x < 0
                  || elementRectangle.topLeft.y < 0) {
            DragManager.showFeedback(DragManager.NONE);
            acceptDrop = false;
          } else {
            DragManager.showFeedback(DragManager.COPY);
            acceptDrop = true;
            //Check collisions
            for each (var elementView:AbstractElementView in diagramView.getChildren()) {
              if ((elementView is IFlowNodeView) &&
                  IFlowNodeView(elementView).getCollisionRectangle(this).intersects(elementRectangle)) {
                DragManager.showFeedback(DragManager.NONE);
                acceptDrop = false;
                return;
              }
            }
          }
        }
      }
    }


    protected override function diagramViewDrop(event:DragEvent):void {
      if (acceptDrop) {
        var targetX:int = event.localX;
        var targetY:int = event.localY;
        if (event.dragSource.hasFormat("paletteItem")) {
          var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
          targetX -= elemDnDModel.deltaX;
          targetY -= elemDnDModel.deltaY;
        }
        var dataModel:AbstractElementWithPosition = null;
        // get the dropped object
        if (event.dragInitiator is DataObjectPaletteView) {
          dataModel = new ExpressionDataObjectModel();
        } else if (event.dragInitiator is DataInputPaletteView) {
          dataModel = new ExpressionDataInputModel();
        } else if (event.dragInitiator is LiteralPaletteView) {
          dataModel = new LiteralModel();
        } else if (event.dragInitiator is ExpressionOperationPaletteView) {
          dataModel = new ExpressionOperationModel();
        }
        if (dataModel != null) {
          new DiagramModificationEvent(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD,
                  diagramEditorModel, dataModel, targetX, targetY).dispatch();
        }
      }
    }

    private function handleMenuClick(menuEvent:MenuEvent):void {
      var menuBarItem:MenuBarItem = menuEvent.item as MenuBarItem;
      var property:String = "";
      if (menuBarItem.models[0] is AbstractExpressionElementModel) {
        if (menuBarItem.models[0] is ExpressionDataObjectModel) {
          property = "dataObjectModel";
        } else if (menuBarItem.models[0] is ExpressionDataInputModel) {
          property = "dataInputModel";
        }
        var childNodeSelector:String = null;
        if (menuBarItem.models.length == 3) {
          childNodeSelector = String(menuBarItem.models[2]);
        }
        //Change selector
        new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, this.dataMappingEditorModel,
                menuBarItem.models[0], "childNodeSelector", childNodeSelector).dispatch();

        new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, this.diagramEditorModel,
                menuBarItem.models[0], property, menuBarItem.models[1]).dispatch();
      } else if (menuBarItem.models[0] is ExpressionOperationModel) {
        // update operation type
        new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, this.dataMappingEditorModel,
                menuBarItem.models[0], "type", menuBarItem.models[1]).dispatch();

      }
    }

    private function createMenu(elementModel:AbstractElementWithPosition, menuContent:ListCollectionView):void {
      var dataListProvider:ListCollectionView = new ArrayCollection();

      var targetX:int = elementModel.x;
      var targetY:int = elementModel.y;
      var elementView:AbstractElementView = this._diagramView.getElementViewByModel(elementModel);
      if (elementView != null) {
        targetX += elementView.width;
      }
      var menuPosition:Point = new Point(targetX, targetY);
      dataListProvider.addAll(menuContent);
      var menu:PatchedScrollableArrowMenu = PatchedScrollableArrowMenu.createMenu(this, dataListProvider);
      menu.verticalScrollPolicy = ScrollPolicy.OFF;
      menu.arrowScrollPolicy = ScrollPolicy.AUTO;
      menu.scrollSpeed = 120;
      menu.addEventListener(MenuEvent.ITEM_CLICK, function (menuEvent:MenuEvent):void {
        var menuBarItem:MenuBarItem = menuEvent.item as MenuBarItem;
        if (menuBarItem != null) {
          if (menuBarItem.handler != null) {
            menuBarItem.handler(menuEvent);
          }
        }

      });
      //Ensure that there is at least 60px to display the menu
      //This should be enough to display both up and down arrows and one element
      var globalDropPoint:Point = this.localToGlobal(menuPosition);
      var menuYDelta:int = Application(Application.application).height - globalDropPoint.y;
      if (menuYDelta < 60) {
        globalDropPoint.y -= 60 - menuYDelta;
      }
      menu.show(globalDropPoint.x, globalDropPoint.y);
    }

    private function getDataObjectMenuContent(targetExpressionDataObjectModel:ExpressionDataObjectModel):IMap {
      var menuContentMap:IMap = new BasicMap();

      for each (var dataObjectModel:DataObjectModel in DesignerModelLocator.getInstance().selectedProcessEditor.processModel.dataObjects.values) {
        // Add data object to menu
        if (!menuContentMap.containsKey(dataObjectModel.name)) {
          var simpleMenuItem:MenuBarItem = new MenuBarItem(dataObjectModel.name, true, this.handleMenuClick,
                  null, null, false, "normal", [targetExpressionDataObjectModel, dataObjectModel]);
          menuContentMap.put(dataObjectModel.name, simpleMenuItem);
        }
        // If the dataObject is a sequence, add sub menu items
        if (dataObjectModel.type is XSDSequenceType ||
            dataObjectModel.type is XSDElementModel && XSDElementModel(dataObjectModel.type).type is XSDSequenceType) {
          var sequenceElements:ListCollectionView;
          if (dataObjectModel.type is XSDSequenceType) {
            sequenceElements = XSDSequenceType(dataObjectModel.type).elements
          } else {
            sequenceElements = XSDSequenceType(XSDElementModel(dataObjectModel.type).type).elements;
          }
          // Create sub menu for the complete dataObject or sub parts
          var parentMenuItem:MenuBarItem = menuContentMap.get(dataObjectModel.name) as MenuBarItem;
          if (!(parentMenuItem is MenuBarItemWithChildren)) {
            parentMenuItem = new MenuBarItemWithChildren(dataObjectModel.name, true);
            menuContentMap.put(dataObjectModel.name, parentMenuItem);
            //Set parent element as a child of itself as parent menu can't be clicked
            MenuBarItemWithChildren(parentMenuItem).children.addItem(
                    new MenuBarItem(dataObjectModel.name, true, this.handleMenuClick,
                            null, null, false, "normal", [targetExpressionDataObjectModel, dataObjectModel]));
            MenuBarItemWithChildren(parentMenuItem).children.addItem(
                    new MenuBarItem(dataObjectModel.name, true, null,
                            null, null, false, "separator"));
          }
          // Add sub parts of the data object to the menu
          for each (var dataTypeModel:DataTypeModel in sequenceElements) {
            // Get data object
            var childMenuItem:MenuBarItem = new MenuBarItem(
                    dataObjectModel.name + "." +
                    dataTypeModel.QName.localPart,
                    true, this.handleMenuClick,
                    null, null, false, "normal", [targetExpressionDataObjectModel, dataObjectModel, dataTypeModel.QName]);
            MenuBarItemWithChildren(parentMenuItem).children.addItem(childMenuItem);
          }
        }
      }
      return menuContentMap;
    }

    private function getDataInputMenuContent(targetExpressionDataInputModel:ExpressionDataInputModel):IMap {
      var menuContentMap:IMap = new BasicMap();

      for each (var dataInputModel:DataInputModel in IBPMNElementWithData(DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.getItemAt(0)).dataInputs) {
        if (!DataUtil.isDataPresent(dataMappingEditorModel.dataMappingModel, dataInputModel)) {
          // Add data input to menu
          if (!menuContentMap.containsKey(dataInputModel.name)) {
            var simpleMenuItem:MenuBarItem = new MenuBarItem(dataInputModel.name, true, this.handleMenuClick,
                    null, null, false, "normal", [targetExpressionDataInputModel, dataInputModel]);
            menuContentMap.put(dataInputModel.name, simpleMenuItem);
          }
        }
        // If the dataInput is a sequence, add sub menu items
        if (dataInputModel.type is XSDSequenceType ||
            dataInputModel.type is XSDElementModel && XSDElementModel(dataInputModel.type).type is XSDSequenceType) {
          var sequenceElements:ListCollectionView;
          if (dataInputModel.type is XSDSequenceType) {
            sequenceElements = XSDSequenceType(dataInputModel.type).elements
          } else {
            sequenceElements = XSDSequenceType(XSDElementModel(dataInputModel.type).type).elements;
          }
          // Create sub menu for the complete dataObject or sub parts
          var dataInputMenuItem:MenuBarItem = menuContentMap.get(dataInputModel.name) as MenuBarItem;
          var parentMenuItem:MenuBarItemWithChildren;
          if (dataInputMenuItem == null || !(dataInputMenuItem is MenuBarItemWithChildren)) {
            parentMenuItem = new MenuBarItemWithChildren(dataInputModel.name, true);
          }
          // Add sub parts of the data input to the menu
          for each (var dataTypeModel:DataTypeModel in sequenceElements) {
            if (!DataUtil.isDataPresent(dataMappingEditorModel.dataMappingModel, dataInputModel, dataTypeModel)) {
              var childMenuItem:MenuBarItem = new MenuBarItem(
                      dataInputModel.name + "." +
                      dataTypeModel.QName.localPart,
                      true, this.handleMenuClick,
                      null, null, false, "normal", [targetExpressionDataInputModel, dataInputModel, dataTypeModel.QName]);
              parentMenuItem.children.addItem(childMenuItem);
            }
          }
          // Add menu only if not empty
          if (parentMenuItem.children.length > 0) {
            //Set parent element as a child of itself as parent menu can't be clicked
            if (dataInputMenuItem != null) {
              parentMenuItem.children.addItemAt(
                      new MenuBarItem(dataInputModel.name, true, this.handleMenuClick,
                              null, null, false, "normal", [targetExpressionDataInputModel, dataInputModel]), 0);
              parentMenuItem.children.addItemAt(
                      new MenuBarItem(dataInputModel.name, true, null,
                              null, null, false, "separator"), 1);
            }
            menuContentMap.put(dataInputModel.name, parentMenuItem);
          }
        }
      }
      return menuContentMap;
    }

    private function getOperationMenuContent(targetExpressionOperationModel:ExpressionOperationModel):IMap {
      var menuContentMap:IMap = new BasicMap();

      for each (var operationType:OperationType in OperationType.constants) {
        // Add operation to menu
        if (!menuContentMap.containsKey(operationType)) {
          var simpleMenuItem:MenuBarItem = new MenuBarItem(ExpressionOperationView.getTypeLabel(operationType), true, this.handleMenuClick,
                  null, null, false, "normal", [targetExpressionOperationModel, operationType]);
          menuContentMap.put(operationType, simpleMenuItem);
        }
      }
      return menuContentMap;
    }

    private function collectionChangeHandler(event:CollectionEvent):void {
      if (event.kind == CollectionEventKind.ADD &&
          event.items.length == 1 &&
          event.items[0] is AbstractElementWithPosition) {
        var addedElement:AbstractElementWithPosition = AbstractElementWithPosition(event.items[0]);
        //By default select first element of each kind
        if (addedElement is ExpressionDataObjectModel) {
          ExpressionDataObjectModel(addedElement).dataObjectModel = DataObjectModel(DesignerModelLocator.getInstance().selectedProcessEditor.processModel.dataObjects.values.getItemAt(0));
          ExpressionDataObjectModel(addedElement).childNodeSelector = null;
          var dataObjectMenuContentMap:IMap = getDataObjectMenuContent(ExpressionDataObjectModel(addedElement));
          if (dataObjectMenuContentMap.length > 1 || dataObjectMenuContentMap.values.getItemAt(0) is MenuBarItemWithChildren) {
            callLater(createMenu, [addedElement, dataObjectMenuContentMap.values]);
          }
        } else if (addedElement is ExpressionDataInputModel) {
          var elementWithData:IBPMNElementWithData =
                  IBPMNElementWithData(DesignerModelLocator.getInstance().selectedProcessEditor.selectedElements.getItemAt(0));
          var dataInputMenuContentMap:IMap = getDataInputMenuContent(ExpressionDataInputModel(addedElement));
          var firstMenuItem:MenuBarItem = MenuBarItem(dataInputMenuContentMap.values.getItemAt(0));
          if (firstMenuItem is MenuBarItemWithChildren) {
            firstMenuItem = MenuBarItem(MenuBarItemWithChildren(firstMenuItem).children.getItemAt(0));
          }
          ExpressionDataInputModel(addedElement).dataInputModel = firstMenuItem.models[1];
          ExpressionDataInputModel(addedElement).childNodeSelector = firstMenuItem.models[2];
          if (dataInputMenuContentMap.length > 1 || dataInputMenuContentMap.values.getItemAt(0) is MenuBarItemWithChildren) {
            callLater(createMenu, [addedElement, dataInputMenuContentMap.values]);
          }
        } else if (addedElement is ExpressionOperationModel) {
          var expressionMenuContentMap:IMap = getOperationMenuContent(ExpressionOperationModel(addedElement));
          callLater(createMenu, [addedElement, expressionMenuContentMap.values]);
        } else if (addedElement is LiteralModel) {
          //Call later literal selector in order to let data mapping view create element view
          //otherwise panel will not have the correct position
          callLater(this.showLiteralSelectorPanel)
        }
      }
    }

    public function dispose():void {
      if (_diagramView != null) {
        DataMappingModel(_diagramView.flowModel).elements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, collectionChangeHandler);
      }
    }

    public function get dataMappingEditorModel():DataMappingEditorModel {
      return DataMappingEditorModel(this.diagramEditorModel);
    }


    override protected function modelSelectedElementsChanged(event:Event):void {
      super.modelSelectedElementsChanged(event);
      if (!(this._diagramEditorModel.selectedElements.length == 1 &&
            this._diagramEditorModel.selectedElements.getItemAt(0) is LiteralModel)) {
        hideLiteralSelectorPanel();
      }
    }

    public function hideLiteralSelectorPanel():void {
      this.literalSelectorPanel.visible = false;
    }

    public function showLiteralSelectorPanel():void {
      if (this.currentState == "default" && this._diagramEditorModel.selectedElements.length == 1 &&
          this._diagramEditorModel.selectedElements.getItemAt(0) is LiteralModel) {
        // get current element
        var literalModemElement:LiteralModel = this._diagramEditorModel.selectedElements.getItemAt(0) as LiteralModel;
        if (literalModemElement != null) {
          // get associated view
          // set context panel attached view (updates the panel content and position)
          this.literalSelectorPanel.attachedView = _diagramView.getElementViewByModel(literalModemElement) as LiteralView;
          // show panel
          this.literalSelectorPanel.setVisible(true);
        }
      } else {
        hideLiteralSelectorPanel();
      }
    }

  }
}
