/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.diagramEditor {
import flash.filters.BitmapFilter;
import flash.filters.BitmapFilterQuality;
import flash.filters.GlowFilter;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.FlowView;

public class DiagramEditorSelectionViewer {

  private var filter:BitmapFilter = new GlowFilter(0x333333, 1, 12, 12, 2, BitmapFilterQuality.MEDIUM);
  private var diagramEditorView:AbstractDiagramEditorView;

  public function DiagramEditorSelectionViewer(diagramEditorView:AbstractDiagramEditorView) {
    this.diagramEditorView = diagramEditorView;
  }

  public function updateViewElement(element:Object, isToBeSelected:Boolean):void {
    if (element is AbstractElement) {
      var elementView:AbstractElementView =
          diagramEditorView.diagramView.getElementViewByModel(element as AbstractElement);
      if (elementView is FlowView) {
        var flowView:FlowView = elementView as FlowView;
        updateFlowView(isToBeSelected, flowView);
      } else {
        if (isToBeSelected) {
          elementView.borderImage.filters = [filter];
        } else {
          elementView.borderImage.filters = [];
        }
      }
    }
  }

  private function updateFlowView(isToBeSelected:Boolean, flowView:FlowView):void {
    if (isToBeSelected) {
      if (diagramEditorView.diagramEditorModel.selectedElements.length == 1) {
        flowView.filters = [];
        flowView.addChild(new FlowSelectionAnchor(flowView, true));
        flowView.addChild(new FlowSelectionAnchor(flowView, false));
      } else {
        flowView.filters = [filter];
        flowView.removeAllChildren();
        flowView.addChild(flowView.canvas);
      }
    } else {
      flowView.filters = [];
      flowView.removeAllChildren();
      flowView.addChild(flowView.canvas);
    }
  }


}
}
