/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


package org.ow2.orchestra.designer.view.processEditor {
import flash.display.DisplayObject;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.collections.ArrayCollection;
import mx.controls.Menu;
import mx.events.MenuEvent;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.bpmn.model.process.PoolModel;
import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;
import org.ow2.orchestra.designer.events.FlowEvent;
import org.ow2.orchestra.designer.view.diagramEditor.*;

public class ProcessEditorCreatingFlowStateHelper extends DiagramEditorCreatingFlowStateHelper {

  public function ProcessEditorCreatingFlowStateHelper(diagramEditorView:AbstractDiagramEditorView, flowStateUtil:DiagramEditorFlowStateUtil) {
    super(diagramEditorView, flowStateUtil);
  }

  /**
   * handler for mouse click events when this view is in the "creating flow" state.
   * Select the element under the mouse cursor and create a flow to this element.
   * The view goes back to the "default" state.
   */
  override protected function creatingFlowMouseClick(event:MouseEvent):void {
    // Get mouse event position. The coordinates are relative to the target element
    var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
    // Transform the coordinates to be relative to this view
    var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

    // check if mouse is on a element view or on the canvas.
    trace("creatingFlowMouseClick");

    if (_flowStateUtil.allowFlowTarget(event)) {
      // find list of elements that are contained under the mouse cursor
      var listSelectedElements:ArrayCollection = _flowStateUtil.getElementUnderMouse(event);
      if (listSelectedElements.length == 1) {
        //Dispatch creating flow event in order to create flow from source element to target element
        var targetElement:AbstractElementWithPosition =
            listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
        // get the source element of flow : this is the only selected element
        var elementSource:AbstractElementWithPosition =
            diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) as AbstractElementWithPosition;


        if (targetElement is PoolModel) {
          var operations:ArrayCollection =
              ((targetElement as PoolModel).interfaces.values.getItemAt(0) as InterfaceModel).operations.values;
          if (operations.length > 1) {
            var menu:Menu = Menu.createMenu(diagramEditorView, operations);
            menu.labelField = "name";
            menu.automationName = "PoolOperationChooserMenu";
            menu.addEventListener(MenuEvent.ITEM_CLICK, function(menuEvent:Event):void {
              new ElementPropertyChangeEvent(
                  ElementPropertyChangeEvent.PROPERTY_CHANGED, diagramEditorView.diagramEditorModel,
                  elementSource,
                  "operation",
                  (menuEvent as MenuEvent).item
              ).dispatch();
            });
            menu.show(diagramEditorView.localToGlobal(mousePoint).x, diagramEditorView.localToGlobal(mousePoint).y);
          }
          if (operations.length > 0) {
            new ElementPropertyChangeEvent(
                ElementPropertyChangeEvent.PROPERTY_CHANGED, diagramEditorView.diagramEditorModel,
                elementSource,
                "operation",
                operations.getItemAt(0)
            ).dispatch();
          }
        } else {
          new FlowEvent(
              FlowEvent.FLOW_CREATION,
              elementSource,
              targetElement,
              diagramEditorView.diagramEditorModel
          ).dispatch();
        }
      }
    }
    // go to default state
    diagramEditorView.currentState = "default";
  }
}
}
