/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.processEditor {
import flash.events.Event;
import flash.geom.Rectangle;

import mx.binding.utils.ChangeWatcher;
import mx.events.DragEvent;
import mx.managers.DragManager;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.process.EventModel;
import org.ow2.orchestra.designer.bpmn.model.process.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.process.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.process.TaskModel;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.IFlowNodeView;
import org.ow2.orchestra.designer.bpmn.views.process.ProcessPoolView;
import org.ow2.orchestra.designer.bpmn.views.process.ProcessView;
import org.ow2.orchestra.designer.events.DiagramModificationEvent;
import org.ow2.orchestra.designer.model.ElementPaletteDnDModel;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.DesignerUtil;
import org.ow2.orchestra.designer.view.diagramEditor.AbstractDiagramEditorView;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorCreatingElementStateHelper;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorCreatingFlowStateHelper;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorDefaultStateHelper;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorFlowStateUtil;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorMovingFlowStateHelper;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorMovingStateHelper;
import org.ow2.orchestra.designer.view.diagramEditor.DiagramEditorSelectingStateHelper;
import org.ow2.orchestra.designer.view.palette.AbstractElementPalette;
import org.ow2.orchestra.designer.view.palette.bpmn.BpmnPalette;
import org.ow2.orchestra.designer.view.palette.bpmn.EventPaletteView;
import org.ow2.orchestra.designer.view.palette.bpmn.GatewayPaletteView;
import org.ow2.orchestra.designer.view.palette.bpmn.TaskPaletteView;
import org.ow2.orchestra.designer.vo.WsdlService;

/**
 *
 * @author hallouin-s
 *
 * This class implements @see(IFocusManagerComponent) to gets the focus
 * on click. This will be used to intercept some events like Keybord events.
 */
public class ProcessEditorView extends AbstractDiagramEditorView {

  /**
   * The source for "palette" image data binding.
   */
  [Embed(source="/../assets/icon/container/bpmn.gif")]
  public static const processIcon:Class;

  internal var processView:ProcessView;

  private var changeWatcher:ChangeWatcher = null;

  /**
   *
   * @param processEditorModel
   */
  public function ProcessEditorView(processEditorModel:ProcessEditorModel) {
    super(processEditorModel);

    onProcessModelChange();

    ChangeWatcher.watch(processEditorModel, "processModel", onProcessModelChange);

    this.icon = processIcon;

    this.automationName =
        "processEditorView-" + processEditorModel.fileNavigatorProcess.project.name + ">"
            + processEditorModel.fileNavigatorProcess.name;
  }

  override protected function createDiagramEditorFlowStateUtil():DiagramEditorFlowStateUtil {
    return new ProcessEditorSequenceFlowStateUtil(this);
  }

  override protected function createDiagramEditorCreatingFlowStateHelper(diagramStateUtil:DiagramEditorFlowStateUtil):DiagramEditorCreatingFlowStateHelper {
    return new ProcessEditorCreatingFlowStateHelper(this, diagramStateUtil);
  }

  private function onProcessModelChange(event:Event = null):void {
    var backgroundViewIndex:int = 0;
    if (processView != null) {
      backgroundViewIndex = this.getChildIndex(processView);
      this.removeChildAt(backgroundViewIndex);
    }
    processView = new ProcessView(this.processEditorModel.processModel);
    processView.percentHeight = 100;
    processView.percentWidth = 100;
    this.addChildAt(processView, backgroundViewIndex);
    _diagramView = processView;

    this.label = this.processEditorModel.processModel.name;

    _viewXOffset = ProcessPoolView.POOL_HEADER_WIDTH;
    if (changeWatcher != null && changeWatcher.isWatching()) {
      changeWatcher.unwatch();
    }
    changeWatcher = ChangeWatcher.watch(this.processEditorModel.processModel, "y", updateYOffset);
  }

  private function updateYOffset(event:Event = null):void {
    _viewYOffset = this.processEditorModel.processModel.y;
  }

  protected override function diagramViewDragEnter(event:DragEvent):void {
    this.dragEnterValid = false;
    this.acceptDrop = false;
    // Accept object
    if (event.dragInitiator is AbstractElementPalette &&
        AbstractElementPalette(event.dragInitiator).parentPalette is BpmnPalette) {
      DragManager.acceptDragDrop(this);
      DragManager.showFeedback(DragManager.COPY);
      this.acceptDrop = true;
      this.dragEnterValid = true;
    } else if (event.dragSource.hasFormat("items")) {
      var items:Array = event.dragSource.dataForFormat("items") as Array;
      if (items.length == 1) {
        var item:Object = items[0];
        if (item is WsdlService) {
          DragManager.acceptDragDrop(this);
          DragManager.showFeedback(DragManager.COPY);
          this.acceptDrop = true;
          this.dragEnterValid = false;
        }
      }
    }
  }

  /** Object drag handler */
  protected override function diagramViewDragOver(event:DragEvent):void {
    if (dragEnterValid) {
      if (event.dragInitiator is AbstractElementPalette) {
        var elementRectangle:Rectangle;
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        elementRectangle = elemDnDModel.image.getRect(this);
        if (elementRectangle.topLeft.x < ProcessPoolView.POOL_HEADER_WIDTH
            || elementRectangle.topLeft.y < processView.processModel.y) {
          DragManager.showFeedback(DragManager.NONE);
          acceptDrop = false;
        } else {
          DragManager.showFeedback(DragManager.COPY);
          acceptDrop = true;
          //Check collisions
          for each (var elementView:AbstractElementView in processView.getChildren()) {
            if ((elementView is IFlowNodeView) &&
                IFlowNodeView(elementView).getCollisionRectangle(this).intersects(elementRectangle)) {
              DragManager.showFeedback(DragManager.NONE);
              acceptDrop = false;
              return;
            }
          }
        }
      } else {
        // Accept object
        if (event.localX < ProcessPoolView.POOL_HEADER_WIDTH
            || event.localY < processView.processModel.y) {
          DragManager.showFeedback(DragManager.NONE);
          acceptDrop = false;
        } else {
          DragManager.showFeedback(DragManager.COPY);
          acceptDrop = true;
        }
      }
    }
  }


  protected override function diagramViewDrop(event:DragEvent):void {
    if (acceptDrop) {
      var dataModel:AbstractElementWithPosition = null;
      // get the dropped object
      if (event.dragInitiator is GatewayPaletteView) {
        dataModel = new GatewayModel();
      } else if (event.dragInitiator is EventPaletteView) {
        dataModel = new EventModel();
        dataModel.name = "event";
      } else if (event.dragInitiator is TaskPaletteView) {
        dataModel = new TaskModel();
        dataModel.name = "task";
      } else if (event.dragSource.hasFormat("items")) {
        // Test if drag from the WSDL navigator
        var items:Array = event.dragSource.dataForFormat("items") as Array;
        if (items.length == 1) {
          var item:Object = items[0];
          if (item is WsdlService) {
            // Create a new pool
            dataModel = new PoolModel();
            dataModel.name = (item as WsdlService).name;

            var iface:InterfaceModel = DesignerUtil.copy((item as WsdlService).interfaceModel) as InterfaceModel;
            iface.poolModel = dataModel as PoolModel;
            (dataModel as PoolModel).interfaces.put(iface.name, iface);
            (dataModel as PoolModel).wsdlNamespace = (item as WsdlService).namespace;
            (dataModel as PoolModel).wsdlPath = (item as WsdlService).wsdlPath;
          }
        }
      }

      var targetX:int = event.localX;
      var targetY:int = event.localY;
      if (event.dragSource.hasFormat("paletteItem")) {
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        targetX -= elemDnDModel.deltaX;
        targetY -= elemDnDModel.deltaY;
      }


      // if a new element is added, generate a process modification event.
      if (dataModel != null) {
        new DiagramModificationEvent(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD,
            this.processEditorModel, dataModel,
            targetX, targetY).dispatch();
      }
    }
  }

  /**
   *
   * @return
   */
  public function get processEditorModel():ProcessEditorModel {
    return ProcessEditorModel(this.diagramEditorModel);
  }

}
}
