/*******************************************************************************
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/

/**
 * Created by porcherg
 * Date: 05/05/11
 * Time: 13:44
 */
package org.ow2.orchestra.designer.view.wsdlNavigator {
  import flash.events.MouseEvent;

  import mx.core.Application;
  import mx.core.IFlexDisplayObject;
  import mx.events.FlexEvent;
  import mx.managers.PopUpManager;

  import org.act.od.impl.other.Localizator;
  import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessageModel;
  import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessagePartModel;
  import org.ow2.orchestra.designer.bpmn.model.operations.OperationModel;
  import org.ow2.orchestra.designer.events.WsdlNavigatorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.vo.WsdlService;

  public class WsdlNavigatorView extends WsdlNavigatorUI {
    /**
     * The source for "uddi" image data binding.
     */
    [Embed(source="/../assets/icon/container/uddi.gif")]
    public static const uddiIcon:Class;


    public function WsdlNavigatorView() {
      this.addEventListener(FlexEvent.CREATION_COMPLETE, onCreationComplete);
      this.icon = uddiIcon;

      this.toolTip = Localizator.getInstance().getText('wsdlNavigator.label');
    }

    private function onCreationComplete(event:FlexEvent):void {
      addButton.addEventListener(MouseEvent.CLICK, addButtonClicked);
      new WsdlNavigatorEvent(WsdlNavigatorEvent.FETCH_MODEL).dispatch();
      this.servicesList.dataProvider = DesignerModelLocator.getInstance().availableServices;
      this.servicesList.dataTipFunction = serviceDataTip;
    }

    private function serviceDataTip(item:WsdlService):String {
      var res:String = "Name: " + item.name + "\n Namespace: " + item.namespace + "\n Operations: \n";
      for each(var op:String in item.interfaceModel.operations.keySet) {
        res = res + "   " + op + "\n";
        var opModel:OperationModel = item.interfaceModel.operations.get(op);
        if (opModel.inMessageType is WSDLMessageModel) {
          res = res + "    in message parts:\n";
          for (var inMessagePartIndex:int = 0; inMessagePartIndex < (opModel.inMessageType as WSDLMessageModel).parts.length; inMessagePartIndex++) {
            var inMessagePart:WSDLMessagePartModel = ((opModel.inMessageType as WSDLMessageModel).parts.getItemAt(inMessagePartIndex) as WSDLMessagePartModel);
            res = res + "     " + inMessagePart.partName + "\n";
          }
        }
        if (opModel.outMessageType != null && opModel.outMessageType is WSDLMessageModel) {
          res = res + "    out message parts:\n";
          for (var outMessagePartIndex:int = 0; outMessagePartIndex < (opModel.outMessageType as WSDLMessageModel).parts.length; outMessagePartIndex++) {
            var outMessagePart:WSDLMessagePartModel = ((opModel.outMessageType as WSDLMessageModel).parts.getItemAt(outMessagePartIndex) as WSDLMessagePartModel);
            res = res + "     " + outMessagePart.partName + "\n";
          }
        }
      }
      return res;
    }

    private function addButtonClicked(event:MouseEvent):void {
      var uploadView:IFlexDisplayObject = PopUpManager.createPopUp(Application.application as Application, UploadView, true);
      PopUpManager.centerPopUp(uploadView);
    }

  }
}
